from ...views import UpdateProfileView, ChangeEmailView, ChangePhoneView
from django.urls import path

urlpatterns = [
    path('update-profile/', UpdateProfileView.as_view(), name='update-profile'),
    path('change-email/', ChangeEmailView.as_view(), name='change-email'),
    path('change-phone/', ChangePhoneView.as_view(), name='change-phone'),
]



from django import forms
from django.contrib.auth import get_user_model, authenticate
from django.contrib.auth.forms import UserChangeForm as BaseUserChangeForm
from django.contrib.auth.forms import UserCreationForm as BaseUserCreationForm
from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

from .verify_phone import VerifyPhone

UserModel = get_user_model()


class UserCreationForm(BaseUserCreationForm):
    class Meta(BaseUserCreationForm.Meta):
        model = UserModel


class RegisterForm(UserCreationForm):
    username = forms.CharField(
        required=True,
        max_length=100, help_text=_("Required, please provide your username"))

    email = forms.EmailField(
        required=True,
        max_length=100, help_text=_("Required, please provide your email"))

    phone = forms.CharField(
        required=True,
        max_length=100, help_text=_("Required, please provide your phone number"))

    class Meta:
        model = UserModel
        fields = ("username", 'email', 'phone', 'password1', 'password2')

    def clean_phone(self):
        phone = self.cleaned_data['phone']
        if UserModel.objects.filter(phone=phone).exists():
            raise ValidationError("Phone already exists", code="unique")
        return phone

    def clean_email(self):
        email = self.cleaned_data['email']
        if UserModel.objects.filter(email=email).exists():
            raise ValidationError("Email already exists", code="unique")
        return email


class MultipleLoginForm(forms.ModelForm):
    password = forms.CharField(
        label=_("Password"),
        required=True,
        strip=False,
        widget=forms.PasswordInput(attrs={'autocomplete': 'current-password'}),
    )
    remember_me = forms.BooleanField(
        required=False,
        label=_('Remember Me'),
        initial=False
    )

    error_messages = {
        'invalid_login': _(
            "Please enter a correct credentials. Note that "
            "fields may be case-sensitive."
        ),
        'inactive': _("This account is inactive."),
        "invalid_credentials": _("Please enter a correct credentials"),
    }

    class Meta:
        model = UserModel
        fields = [*UserModel.AUTHENTICATION_FIELDS]

    def __init__(self, request=None, *args, **kwargs):
        self.request = request
        self.user_cache = None
        super(MultipleLoginForm, self).__init__(*args, **kwargs)
        for field in self.fields:
            self.fields[field].required = False

    def get_user(self):
        return self.user_cache

    def clean(self):
        password = self.cleaned_data.pop('password')
        remember_me = self.cleaned_data.pop('remember_me')

        if not password:
            raise ValidationError(self.error_messages['invalid_credentials'],
                                  code='invalid_credentials')

        login_by = next((key for key in list(self.cleaned_data.keys()) if self.cleaned_data[key]), None)

        if login_by not in UserModel.AUTHENTICATION_FIELDS:
            raise ValidationError(
                self.error_messages['invalid_credentials'],
                code='invalid_credentials')

        credentials = {"password": password}

        credentials.update({login_by: self.cleaned_data[login_by]})
        if login_by and password:
            self.user_cache = authenticate(request=self.request, **credentials)
            if not self.user_cache:
                raise ValidationError(
                    self.error_messages['invalid_login'],
                    code='invalid_login')
            if not remember_me and self.request:
                self.request.session.set_expiry(0)
                self.request.session.modified = True
            else:
                if not self.user_cache.is_active:
                    raise ValidationError(self.error_messages['inactive'], code='inactive')

        return self.cleaned_data


class VerifyPhoneForm(forms.Form):
    code = forms.CharField(max_length=6, min_length=6)

    def __init__(self, *args, **kwargs):
        self.user = kwargs.pop('user', None)
        super(VerifyPhoneForm, self).__init__(*args, **kwargs)
        self.fields['code'].label = "message sent to phone number: {}".format(self.user.phone)

    def clean(self):
        code = self.cleaned_data.get('code')

        success = VerifyPhone().check(self.user.phone, code)
        if not success:
            raise ValidationError(_("The Provided code is Properly invalid"), code='invalid_code')

        return self.cleaned_data


# profile forms


class UserChangeForm(BaseUserChangeForm):
    class Meta(BaseUserChangeForm.Meta):
        model = UserModel
        fields = ('username', 'first_name', 'last_name')

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class UpdateUserDataForm(UserChangeForm):
    class Meta:
        model = UserModel
        fields = ('first_name', 'last_name')

    def clean_first_name(self):
        first_name = self.cleaned_data.get('first_name', None)
        if not first_name:
            raise ValidationError(_("Please Enter the first name"), code="required")

        return first_name

    def clean_last_name(self):
        last_name = self.cleaned_data.get('last_name')
        if not last_name:
            raise ValidationError("Please Enter the last name", code="required")

        return last_name


class UpdateEmailForm(forms.Form):
    new_email = forms.EmailField(required=True, label=_("New Email"))
    password = forms.CharField(required=True,
                               label=_("Password"),
                               strip=False,
                               widget=forms.PasswordInput(attrs={'autocomplete': 'current-password'}),
                               )

    error_messages = {
        'invalid_login': _(
            "Please enter a correct phone and password. Note that both "
            "fields may be case-sensitive."
        )
    }

    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(UpdateEmailForm, self).__init__(*args, **kwargs)

    def clean_new_email(self):
        old_email = self.user.email
        email = self.cleaned_data.get('new_email')

        if UserModel.objects.filter(email=email).exists():
            raise ValidationError(_("Email already exists"), code="unique")

        if old_email == email:
            raise ValidationError(_("Please Enter the new Email"), code="email_mismatch")

        return email

    def clean_password(self):
        username = self.user.email
        password = self.cleaned_data.get('password')

        credentials = {"password": password}
        credentials.update({UserModel.EMAIL_FIELD: username})

        if not authenticate(**credentials):
            raise ValidationError(_("Please Enter Valid Password"), code="invalid")

        return password

    def save(self, commit=True):
        email = self.cleaned_data["new_email"]
        self.user.email = email
        if commit:
            self.user.save()
        return self.user


class UpdatePhoneNumberForm(forms.Form):
    new_phone = forms.CharField(required=True)
    password = forms.CharField(
        label=_("Your Password"),
        strip=False,
        widget=forms.PasswordInput(attrs={'autocomplete': 'current-password'}),
    )

    error_messages = {
        'invalid_login': _(
            "Please enter a correct phone and password. Note that both "
            "fields may be case-sensitive."
        )
    }

    def __init__(self, user=None, *args, **kwargs):
        self.user = user
        super(UpdatePhoneNumberForm, self).__init__(*args, **kwargs)
        self.phone_field = UserModel._meta.get_field(UserModel.REQUIRED_FIELDS[0])

    def clean_new_phone(self):
        phone = self.cleaned_data['new_phone']
        if UserModel.objects.filter(phone=phone).exists():
            raise ValidationError("Phone already exists", code="unique")
        return phone

    def clean(self):
        phone = self.user.phone
        password = self.cleaned_data.get('password')
        credentials = {"password": password}

        credentials.update({"phone": phone})

        if phone and password:
            authenticated_user = authenticate(**credentials)

        return self.cleaned_data

    def save(self, commit=True):
        phone = self.cleaned_data["new_phone"]
        self.user.phone = phone
        if commit:
            self.user.save()
        return self.user



from ..verify_phone import VerifyPhone, VerifyPhoneServiceAbstract
from django.test import TestCase, override_settings


class VerifyPhoneSendTestCase(TestCase):
    def setUp(self):
        self.phone = "201002536987"

    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_calls_the_select_service_send(self):
        success = VerifyPhone().send(phone=self.phone)
        self.assertTrue(success)


class VerifyPhoneCheckTestCase(TestCase):
    def setUp(self):
        self.phone = "201002536987"
        self.code = "777777"

    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_calls_the_selected_service_check(self):
        success = VerifyPhone().check(phone=self.phone, code=self.code)
        self.assertTrue(success)

    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_returns_false_if_phone_or_code_is_not_correct(self):
        success = VerifyPhone().check(phone="201063598876", code=self.code)
        self.assertFalse(success)


class VerifyPhoneGetServiceClassTestCase(TestCase):
    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_returns_subclass_of_verify_phone_abstract(self):
        class_ = VerifyPhone().get_service_class()
        self.assertIsInstance(class_, VerifyPhoneServiceAbstract)



from django import forms
from django.contrib.auth import get_user_model
from django.contrib.auth.forms import UserCreationForm
from django.contrib.sessions.middleware import SessionMiddleware
from django.test import TestCase, RequestFactory
from django.test import override_settings
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _

from .factories import UserFactory
from ..forms import MultipleLoginForm, RegisterForm, VerifyPhoneForm, UpdateUserDataForm, UpdateEmailForm, \
    UpdatePhoneNumberForm

UserModel = get_user_model()


def get_default_user_data():
    return {
        "email": "test@test.test",
        "username": "Test User",
        "phone": "+201005263988",
        "password1": "newTESTPasswordD",
        "password2": "newTESTPasswordD",
        "first_name": "Test",
        "last_name": "User",
        "gender": "male",
        "birthdate": now(),
    }


def check_required(cls, key):
    data = {**cls.DEFAULT_DATA}
    data.pop(key, None)
    instance = cls.Form(data=data)
    cls.assertFalse(instance.is_valid())
    cls.assertEquals(instance.errors.as_data()[key][0].code, 'required')


def check_unique(cls, key, value):
    data = {**cls.DEFAULT_DATA}
    data.update({key: value})
    instance = cls.Form(data=data)
    cls.assertFalse(instance.is_valid())
    cls.assertEquals(instance.errors.as_data()[key][0].code, 'unique')


class RegistrationFormStructureTestCase(TestCase):
    def setUp(self):
        self.form = RegisterForm()

    def test_it_extends_django_Form(self):
        self.assertTrue(issubclass(RegisterForm, UserCreationForm))

    def test_it_has_email_field(self):
        self.assertIn('email', self.form.fields)

    def test_email_field_is_instance_of_email_field(self):
        self.assertIsInstance(self.form.fields['email'], forms.CharField)

    def test_email_field_max_length_is_100(self):
        self.assertEquals(self.form.fields['email'].max_length, 100)

    def test_email_field_help_text(self):
        self.assertEquals(self.form.fields['username'].help_text, _("Required, please provide your username"))

    def test_it_has_username_field(self):
        self.assertIn('username', self.form.fields)

    def test_username_field_is_instance_of_char_field(self):
        self.assertIsInstance(self.form.fields['username'], forms.CharField)

    def test_user_name_field_is_required(self):
        self.assertTrue(self.form.fields['username'].required)

    def test_username_field_max_length_is_100(self):
        self.assertEquals(self.form.fields['username'].max_length, 100)

    def test_username_field_help_text(self):
        self.assertEquals(self.form.fields['username'].help_text, _("Required, please provide your username"))

    def test_it_has_phone_field(self):
        self.assertIn('phone', self.form.fields)

    def test_phone_field_is_instance_of_char_field(self):
        self.assertIsInstance(self.form.fields['phone'], forms.CharField)

    def test_phone_field_is_required(self):
        self.assertTrue(self.form.fields['phone'].required)

    def test_phone_field_max_length_is_100(self):
        self.assertEquals(self.form.fields['phone'].max_length, 100)

    def test_phone_field_help_text(self):
        self.assertEquals(self.form.fields['phone'].help_text, _("Required, please provide your phone number"))


class RegisterFormValidationTest(TestCase):
    def setUp(self):
        self.DEFAULT_DATA = get_default_user_data()
        self.Form = RegisterForm
        UserModel.objects.create_user(
            email='first@aol.com', username="First User", first_name="First",
            last_name="User", phone="+201005263987")

    def test_username_is_required(self):
        check_required(self, 'username')

    def test_phone_is_required(self):
        check_required(self, 'phone')

    def test_phone_is_unique(self):
        check_unique(self, "phone", "+201005263987")

    def test_email_is_required(self):
        check_required(self, 'email')

    def test_email_is_unique(self):
        check_unique(self, "email", "first@aol.com")

    def test_password1_is_required(self):
        check_required(self, 'password1')

    def test_password2_is_required(self):
        check_required(self, 'password2')

    def test_password2_mismatch(self):
        data = self.DEFAULT_DATA
        data.update({
            "password1": "123654789",
            "password2": "123456789"
        })
        form = RegisterForm(data=data)
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors.as_data()["password2"][0].code, 'password_mismatch')


class MultipleLoginFormStructureTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory()
        self.form = MultipleLoginForm(request=self.request)

    def test_it_extends_django_model_form(self):
        self.assertTrue(issubclass(MultipleLoginForm, forms.ModelForm))

    def test_it_has_cached_user(self):
        self.assertTrue(hasattr(self.form, 'user_cache'))

    def test_it_has_get_user_method(self):
        self.assertTrue(hasattr(self.form, 'get_user') and callable(self.form.get_user))

    def test_get_user_method_returns_cache_user_value(self):
        self.assertEquals(self.form.user_cache, self.form.get_user())

    def test_it_has_password_field(self):
        self.assertIn('password', self.form.fields)

    def test_password_field_is_instance_of_char_field(self):
        self.assertIsInstance(self.form.fields['password'], forms.CharField)

    def test_password_has_label_Password(self):
        self.assertEquals(self.form.fields['password'].label, _('Password'))

    def test_password_field_not_striping_its_value(self):
        form = MultipleLoginForm()
        self.assertFalse(self.form.fields['password'].strip)

    def test_password_widget_is_instance_of_passwordInput(self):
        self.assertIsInstance(self.form.fields['password'].widget, forms.PasswordInput)

    def test_it_has_remember_me_field(self):
        self.assertIn('remember_me', self.form.fields)

    def test_remember_me_field_is_instance_of_charfield(self):
        self.assertIsInstance(self.form.fields['remember_me'], forms.BooleanField)

    def test_remember_me_field_has_label_Remember_Me(self):
        self.assertEquals(self.form.fields['remember_me'].label, _('Remember Me'))

    def test_remember_me_initial_value_is_false(self):
        self.assertEquals(self.form.fields['remember_me'].initial, False)


class MultipleLoginFormValidationTestCase(TestCase):
    @override_settings(AUTHENTICATION_BACKENDS=["dj_accounts.backends.MultipleAuthenticationBackend"])
    def setUp(self):
        self.request = RequestFactory().get('/')
        middleware = SessionMiddleware()
        middleware.process_request(self.request)
        self.request.session.save()
        self.user = UserFactory()
        self.Form = MultipleLoginForm
        self.user.phone = "01102158610"
        self.user.save()
        self.DEFAULT_DATA = {
            "email": self.user.email,
            "phone": self.user.phone,
            "password": "secret",
            "remember_me": True
        }

    def test_it_fails_if_both_phone_and_email_are_not_provided(self):
        self.DEFAULT_DATA.pop('phone')
        self.DEFAULT_DATA.pop('email')
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals('invalid_credentials', form.errors.as_data()['__all__'][0].code)
        self.assertEquals(_("Please enter a correct credentials"), form.errors.as_data()['__all__'][0].message)

    # def test_password_is_required(self):
    #     check_required(self, 'password')

    def test_it_fails_if_password_is_not_correct(self):
        self.DEFAULT_DATA.update({"password": "12345"})
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals('invalid_login', form.errors.as_data()['__all__'][0].code)
        self.assertEquals(_(
            "Please enter a correct credentials. Note that "
            "fields may be case-sensitive."
        ), form.errors.as_data()['__all__'][0].message)

    def test_it_fails_if_email_is_not_correct(self):
        self.DEFAULT_DATA.pop('phone')
        self.DEFAULT_DATA.update({"email": "aaa@lll.ccc"})
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals('invalid_login', form.errors.as_data()['__all__'][0].code)
        self.assertEquals(_(
            "Please enter a correct credentials. Note that "
            "fields may be case-sensitive."
        ), form.errors.as_data()['__all__'][0].message)

    def test_it_fails_if_phone_is_not_correct(self):
        self.DEFAULT_DATA.pop('email')
        self.DEFAULT_DATA.update({"phone": "56415615"})
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals('invalid_login', form.errors.as_data()['__all__'][0].code)
        self.assertEquals(_(
            "Please enter a correct credentials. Note that "
            "fields may be case-sensitive."
        ), form.errors.as_data()['__all__'][0].message)

    def test_it_authenticates_correct_user_by_phone(self):
        self.DEFAULT_DATA.pop('email')
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        form.is_valid()
        self.assertTrue(form.is_valid())

    def test_it_authenticates_correct_user_by_username(self):
        self.DEFAULT_DATA.pop('phone')
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertTrue(form.is_valid())

    def test_it_saves_authenticated_user_in_user_cache(self):
        self.DEFAULT_DATA.pop('phone')
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        form.is_valid()
        self.assertEquals(form.user_cache, self.user)

    def test_get_user_method_returns_authenticated_user(self):
        self.DEFAULT_DATA.pop('phone')
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        form.is_valid()
        print(form.errors)
        self.assertEquals(form.get_user(), self.user)

    def test_it_sets_session_expiry_to_zero_if_remember_me_is_false(self):
        self.DEFAULT_DATA.pop('phone')
        self.DEFAULT_DATA.update({"remember_me": False})
        form = self.Form(request=self.request, data=self.DEFAULT_DATA)
        self.assertTrue(form.is_valid())
        self.assertIn('_session_expiry', self.request.session)
        self.assertEquals(0, self.request.session['_session_expiry'])


class VerifyPhoneFormTestCase(TestCase):
    def setUp(self):
        self.user = UserFactory(phone="201002536987")
        self.form = VerifyPhoneForm(user=self.user)

    def test_it_extends_django_Form(self):
        self.assertIsInstance(self.form, forms.Form)

    def test_it_has_code_field(self):
        self.assertIn('code', self.form.fields)

    def test_code_label_is_correct(self):
        self.assertEquals(self.form.fields['code'].label, _("message sent to phone number: {}".format(self.user.phone)))

    def test_code_min_length_is_six_characters(self):
        self.assertEquals(self.form.fields['code'].min_length, 6)

    def test_code_max_length_is_six_characters(self):
        self.assertEquals(self.form.fields['code'].max_length, 6)


class VerifyPhoneFormValidationTestCase(TestCase):
    def setUp(self):
        self.user = UserFactory(phone="201002536987")
        self.data = {"code": "777777"}

    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_fails_if_phone_verification_is_not_successful(self):
        self.data.update({"code": "888888"})
        form = VerifyPhoneForm(user=self.user, data=self.data)
        self.assertFalse(form.is_valid())
        self.assertEquals('invalid_code', form.errors.as_data()['__all__'][0].code)
        self.assertEquals(_("The Provided code is Properly invalid"), form.errors.as_data()['__all__'][0].message)

    @override_settings(PHONE_VERIFY_SERVICE="dj_accounts.tests.mocks.MockVerifyService")
    def test_it_passes_if_phone_verification_is_successful(self):
        form = VerifyPhoneForm(user=self.user, data=self.data)
        self.assertTrue(form.is_valid())


# profile tests

class UpdateUserDataFormStructureTestCase(TestCase):
    def setUp(self):
        self.form = UpdateUserDataForm()
        self.DEFAULT_DATA = get_default_user_data()

    def test_form_class_is_subclass_from_model_form(self):
        self.assertTrue(issubclass(UpdateUserDataForm, forms.ModelForm))

    def test_form_has_meta_class(self):
        self.assertTrue(hasattr(self.form, 'Meta'))

    def test_form_model_is_user_model(self):
        self.assertEquals(self.form.Meta.model, UserModel)

    def test_form_fields_are_equal_to_user_model_fields(self):
        self.assertEquals(self.form.Meta.fields, ('first_name', 'last_name'))

    def test_it_has_clean_first_name(self):
        self.assertTrue(hasattr(self.form, 'clean_first_name'))

    def test_it_has_clean_last_name(self):
        self.assertTrue(hasattr(self.form, 'clean_last_name'))

    def test_html_representation_has_first_name_field(self):
        self.assertIn('name="first_name"', str(self.form))

    def test_html_representation_has_last_name_field(self):
        self.assertIn('name="last_name"', str(self.form))


class UpdateUserFormValidationTest(TestCase):
    def setUp(self):
        self.DEFAULT_DATA = get_default_user_data()
        self.Form = UpdateUserDataForm

    def test_update_user_data_form(self):
        form = UpdateUserDataForm(data=self.DEFAULT_DATA)
        self.assertTrue(form.is_valid())

    def test_update_user_data_form_with_empty_first_name(self):
        check_required(self, 'first_name')

    def test_update_user_data_form_with_empty_last_name(self):
        check_required(self, 'last_name')

    def test_it_rais_error_message_if_no_first_name(self):
        self.DEFAULT_DATA.pop('first_name')
        form = UpdateUserDataForm(data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors.as_data()['first_name'][0].code, 'required')

    def test_it_rais_error_message_if_no_last_name(self):
        self.DEFAULT_DATA.pop('last_name')
        form = UpdateUserDataForm(data=self.DEFAULT_DATA)
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors.as_data()['last_name'][0].code, 'required')


class UpdateEmailFormStructureTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory().get('/')
        self.user = UserFactory(email='test@test.test', password='secret')
        self.Form = UpdateEmailForm
        self.DEFAULT_DATA = {
            "new_email": 'new_test@test.test',
            "password": "secret",
        }

    def test_form_class_is_subclass_from_forms_form(self):
        self.assertTrue(issubclass(UpdateEmailForm, forms.Form))

    def test_html_representation_has_new_email_field(self):
        self.assertIn('name="new_email"', str(self.Form()))

    def test_html_representation_has_password_field(self):
        self.assertIn('name="password"', str(self.Form()))

    def test_form_has_error_messages_field(self):
        self.assertTrue(hasattr(self.Form, 'error_messages'))

    def test_it_has_clean_new_email(self):
        self.assertTrue(hasattr(self.Form, 'clean_new_email'))

    def test_it_has_clean_password(self):
        self.assertTrue(hasattr(self.Form, 'clean_password'))

    def test_it_has_save(self):
        self.assertTrue(hasattr(self.Form, 'save'))


class UpdateEmailFormValidationTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get('/')
        self.user = UserFactory(email='test@test.test', password='secret')
        self.Form = UpdateEmailForm
        self.DEFAULT_DATA = {
            "new_email": 'new_test@test.test',
            "password": "secret",
        }

    def test_email_form(self):
        form = UpdateEmailForm(data=self.DEFAULT_DATA, user=self.user)
        self.assertTrue(form.is_valid())

    def test_email_from_throw_error_if_provides_incomplete_data(self):
        form = UpdateEmailForm(data={}, user=self.user)
        self.assertFalse(form.is_valid())

    def test_email_from_throw_error_if_email_is_email_is_already_exists(self):
        default_data = {
            "new_email": 'test@test.test',
            "password": "secret",
        }
        form = UpdateEmailForm(data=default_data, user=self.user)
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors.as_data()['new_email'][0].code, 'unique')

    def test_email_field_throw_error_if_old_email_equal_new_email(self):
        default_data = {
            "new_email": 'test@test.test',
            "password": "secret",
        }
        form = UpdateEmailForm(data=default_data, user=self.user)
        self.assertFalse(form.is_valid())
        self.assertEquals(form.errors.as_data()['new_email'][0].code, 'unique')

    def test_email_from_throw_error_if_email_is_invalid(self):
        data = {
            "new_email": 'fake',
            "password": "secret",
        }
        form = UpdateEmailForm(data=data, user=self.user)
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors.as_data()['new_email'][0].code, 'invalid')

    def test_email_from_throw_error_if_password_is_invalid(self):
        data = {
            "new_email": 'test@test.sdfsd',
            'password': 'fake',
        }

        form = UpdateEmailForm(data=data, user=self.user)
        self.assertFalse(form.is_valid())
        self.assertEqual(form.errors.as_data()['password'][0].code, 'invalid')

    def test_ti_change_user_email(self):
        data = {
            "new_email": 'new@new.com',
            "password": "secret",
        }
        form = UpdateEmailForm(data=data, user=self.user)
        self.assertTrue(form.is_valid())
        form.save()
        self.assertEqual(self.user.email, 'new@new.com')


class UpdatePhoneFormTestCase(TestCase):
    def setUp(self):
        self.request = RequestFactory()
        self.form = UpdatePhoneNumberForm()

    def test_it_extends_django_Form(self):
        self.assertIsInstance(UpdatePhoneNumberForm(), forms.Form)

    def test_it_has_phone_field(self):
        self.assertIn('new_phone', self.form.fields)

    def test_phone_field_is_instance_of_char_field(self):
        self.assertIsInstance(self.form.fields['new_phone'], forms.CharField)

    def test_phone_field_is_required(self):
        self.assertTrue(self.form.fields['new_phone'].required)

    def test_it_has_password_field(self):
        self.assertIn('password', self.form.fields)

    def test_password_field_is_instance_of_char_field(self):
        self.assertIsInstance(self.form.fields['password'], forms.CharField)

    def test_password_has_label_Password(self):
        self.assertEquals(self.form.fields['password'].label, _("Your Password"))

    def test_password_field_not_striping_its_value(self):
        form = UpdateEmailForm()
        self.assertFalse(self.form.fields['password'].strip)

    def test_password_widget_is_instance_of_passwordInput(self):
        self.assertIsInstance(self.form.fields['password'].widget, forms.PasswordInput)


class UpdatePhoneFormValidationTest(TestCase):
    def setUp(self):
        self.request = RequestFactory().get('/')
        self.user = UserFactory(phone="+201005263988")
        self.DEFAULT_DATA = get_default_user_data()
        self.Form = UpdatePhoneNumberForm
        self.DEFAULT_DATA = {
            "new_phone": '+201005263977',
            "password": "secret",
        }

    def test_phone_form(self):
        form = UpdatePhoneNumberForm(data=self.DEFAULT_DATA, user=self.user)
        self.assertTrue(form.is_valid())

    def test_phone_from_throw_error_if_provides_incomplete_data(self):
        form = UpdatePhoneNumberForm(data={}, user=self.user)
        self.assertFalse(form.is_valid())

    def test_phone_from_throw_error_if_new_phone_already_exists(self):
        default_data = {
            "new_phone": '+201005263988',
            "password": "secret",
        }
        form = UpdatePhoneNumberForm(data=default_data, user=self.user)
        self.assertFalse(form.is_valid())



from ..views import ResendPhoneConfirmationView, VerifyPhoneView, PhoneVerificationCompleteView, LoginView, \
    RegisterView, ResendEmailConfirmationLinkView as EmailConfirmationView, ChangeEmailView, ChangePhoneView, \
    UpdateProfileView
from ..views_api import UpdateProfileAPIView, ChangeEmailAPIView, ChangePhoneAPIView, VerifyPhoneAPIView, \
    ResendPhoneConfirmationAPIView, VerifyEmailAPIView, ResendEmailConfirmationLinkAPIView, ChangePasswordAPIView
from django.contrib.auth import views
from django.test import TestCase
from django.urls import resolve, reverse


class SiteUrlsTestCase(TestCase):
    def test_login_url_resolves(self):
        url = reverse('login')
        self.assertEquals(resolve(url).func.view_class, LoginView)

    def test_register_url_resolves(self):
        url = reverse('register')
        self.assertEquals(resolve(url).func.view_class, RegisterView)

    def test_logout_url_resolves(self):
        url = reverse('logout')
        self.assertEquals(resolve(url).func.view_class, views.LogoutView)

    # phone urls
    def test_phone_verify_url_resolves(self):
        url = reverse("verify-phone")
        self.assertEquals(resolve(url).func.view_class, VerifyPhoneView)

    def test_resend_phone_confirmation_url_resolves(self):
        url = reverse("resend_phone_activation")
        self.assertEquals(resolve(url).func.view_class, ResendPhoneConfirmationView)

    def test_phone_verify_complete_url_resolves(self):
        url = reverse("phone-verification-complete")
        self.assertEquals(resolve(url).func.view_class, PhoneVerificationCompleteView)

    # Email Verification
    def test_resend_email_confirmation_link_url_resolves(self):
        url = reverse("resend-email-confirmation")
        self.assertEquals(resolve(url).func.view_class, EmailConfirmationView)

    def test_change_email_url_resolves(self):
        url = reverse("change-email")
        self.assertEquals(resolve(url).func.view_class, ChangeEmailView)

    def test_change_phone_url_resolves(self):
        url = reverse("change-phone")
        self.assertEquals(resolve(url).func.view_class, ChangePhoneView)

    def test_update_profile_url_resolves(self):
        url = reverse("update-profile")
        self.assertEquals(resolve(url).func.view_class, UpdateProfileView)


class APIUrlsTestCase(TestCase):

    # EMAIL
    def test_verify_email_confirmation_url_resolves(self):
        url = reverse('verify_email_api', args=['token', 'email'])
        self.assertEquals(resolve(url).func.view_class, VerifyEmailAPIView)

    def test_resend_email_confirmation_code_url_resolves(self):
        url = reverse('resend_email_activation_api')
        self.assertEqual(resolve(url).func.view_class, ResendEmailConfirmationLinkAPIView)

    # phone verification
    def test_phone_verify_url_resolves(self):
        url = reverse("verify_phone_api")
        self.assertEquals(resolve(url).func.view_class, VerifyPhoneAPIView)

    def test_resend_phone_activation_code_url_resolves(self):
        url = reverse('resend_phone_activation_api')
        self.assertEqual(resolve(url).func.view_class, ResendPhoneConfirmationAPIView)

    # profile
    def test_user_update_profile_info_api_view_url_resolves(self):
        url = reverse('update-profile-api')
        self.assertEqual(resolve(url).func.view_class, UpdateProfileAPIView)

    def test_update_user_email_api_view_url_resolves(self):
        url = reverse('change-email-api')
        self.assertEqual(resolve(url).func.view_class, ChangeEmailAPIView)

    def test_update_user_phone_api_view_url_resolves(self):
        url = reverse('change-phone-api')
        self.assertEqual(resolve(url).func.view_class, ChangePhoneAPIView)

    # password
    def test_password_change_api_view_url_resolves(self):
        url = reverse('change_password_api')
        self.assertEqual(resolve(url).func.view_class, ChangePasswordAPIView)



import importlib
from abc import ABC, abstractmethod

from django.conf import settings


class VerifyPhoneServiceAbstract(ABC):
    class Meta:
        abstract = True

    @abstractmethod
    def send(self, phone):
        pass

    @abstractmethod
    def check(self, phone, code):
        pass


class VerifyPhone:
    def __init__(self):
        self.service = self.get_service_class()

    def send(self, phone):
        return self.service.send(phone)

    def check(self, phone, code):
        return self.service.check(phone, code)

    @staticmethod
    def get_service_class():
        service_name = settings.PHONE_VERIFY_SERVICE
        split_service_name = service_name.split('.')
        class_name = split_service_name[-1:][0]
        module_name = split_service_name[:-1]
        service_class = getattr(importlib.import_module('.'.join(module_name)), class_name)
        return service_class()



from ..verify_phone import VerifyPhoneServiceAbstract


class MockVerifyService(VerifyPhoneServiceAbstract):
    phone = "201002536987"
    code = "777777"

    def send(self, phone):
        return True

    def check(self, phone, code):
        return phone == self.phone and code == self.code


class TestingVerifyService(VerifyPhoneServiceAbstract):
    def send(self, phone):
        return True

    def check(self, phone, code):
        return True
